/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	Written by: Joshua Horacsek
 *	Mon Jan  2 14:55:40 2006
 */

#ifndef __LIBHALO_2__
#define __LIBHALO_2__

#ifdef __cplusplus
extern "C"
{
	
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//If you want slower, more accurate(maybe not?) scanning,
//make sure the following line is uncommented
#define BETTER_SCANNING
#define A1

//If you want the maps to be resigned automatically when closed,
//make sure the following line is uncommented
#define	AUTOMATIC_RESIGN
	
typedef unsigned int uint32;
typedef unsigned short uint16;
typedef unsigned char uint8;

typedef unsigned int dword;
typedef unsigned short word;
typedef unsigned char byte;
/*
Map Structures:
	Map Header:
	Includes alot of important data used in parsing a map.

	Index Header:
	Includes information such as tag count, item count and the primary magic constant.

	Tag Index:
	Index of tags that define the types of items used in the map.
	
	Item Index:
	Index of all the items(meta) in a map which include the items IDs
	(which can be calculated from its tag number), classes, Item Numbers and
	Item Size.
	
*/
typedef struct{
    char header[4];
    uint32 Version; // 8 for Halo 2
    uint32 Filesize; //
    uint32 Zero; //
    uint32 indexOffset;
    uint32 indexSize; // meta start
    uint32 metaSize;
    uint32 NonRawSize; // MetaSize + index Size + BSP Size
    char filename[0x100];
    char builddate[0x20];
    uint32 maptype;
    uint32 unknowns[4];
    uint32 sizeofStrangeFileStrings;
    uint32 offsetToStrangeFileStrings;
    uint32 unknown7;
    uint32 offsetToScriptReference128;
    uint32 scriptReferenceCount;
    uint32 sizeOfScriptReference;
    uint32 offsetToScriptReferenceIndex;
    uint32 offsetToScriptReferenceStrings;
    uint8 unknowns2[36];
    char mapname[36];
    char scenarioname[0x100];
    uint32 unknown2;
    uint32 fileCount;
    uint32 fileTableOffset;
    uint32 fileTableSize;
    uint32 filesIndex;
    uint32 XOR_Checksum;
    uint8  Reserved[1320];
    char footer[4];
} Halo2Map, *_Halo2Map;

typedef struct {
    uint32 indexmagic;	//Primary Magic Consatant Magic
    uint32 tagcount;
    uint32 itemsmagic;
    uint32 scnrid;
    uint32 firstid;
    uint32 unknown1;
    uint32 itemcount;
    uint32 tagstart; // 'tags'
}IndexHeader;

typedef struct {
  char objectClass[4];
  char parentClass[4];
  char grandparentClass[4];
}IndexTag;

typedef struct {
    char tagclass[4];
    uint32 id;
    uint32 offset;
    uint32 size;
}IndexItem; 
/*
Refrences:
	The following structures define refrences in Halo 2 Map file's Meta

	Lone ID:
	One unsigned integer that points to a Tag's ID

	Dependency:
	Contains a Lone ID, and the Class of that Lone ID

	String Indice:
	Contains a string number to the string index. See
	Contains the length of the string pointed to.
*/
typedef unsigned int LoneID;

typedef struct {
	char Class[4];
	LoneID Tag;
}Dependency, *_Dependency;

typedef struct {
	uint16 StringRef;
	uint8 unused;//Actually part of Stringref, but we don't use it... 
	uint8 StringLen;
} StringIndice, *_StringIndice;

typedef struct {
	uint32 ChunkCount;
	uint32 Offset;
} Reflexive, *_Reflexive;


typedef struct {
	StringIndice StringRef;
	uint32 UnicodeLen;
} UnicodeString;


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
typedef struct{
	uint32 Magic;
	uint32 Offset;
	uint32 OffsetToOffset;
	uint32 Size;
	uint32 BSPBlockSize;
	uint32 TagID;
	uint32 Tag;
}BSP_Data;

typedef struct{
	uint32 Magic;
	uint32 Offset;
	uint32 Size;
	uint32 Tag;
} LTMP_Data;

typedef struct{
	uint32 IndexOffset;
	uint32 UnicodeStringTableOffset;
	uint32 USTSize;
	
	uint32 StringCount;
	UnicodeString *StringIndex;
}UnicodeStringTable;
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
typedef struct {
	Halo2Map			*Header;
	IndexHeader			*Index;
	IndexTag			*TagIndex;
	IndexItem			*ItemIndex;
	
	uint32				TagIndexOffset;
	uint32				ItemIndexOffset;
	
	uint32				PrimaryMagic;
	uint32				SecondaryMagic;
	
	char				MapName[36];
	char				scenarioname[0x100];

	uint32				MetaCount;
	uint32				*MetaIndex;	//Ex: MetaTable+MetaIndex[i] will give you a string(Meta's Name)
	char				*MetaTable;	//Where 'i' is an integer between 0 and MetaCount

	uint32				StringCount;
	uint32				*StringIndex; 
	char				*StringTable;
	
	int					LanguageCount;
	UnicodeStringTable 	*UnicLangTable;
	
	int					bspCount;
	BSP_Data			*BSPs;
	
	int					ltmpCount;
	LTMP_Data			*LTMPs;
	
	char				*MapPath;
	FILE				*Stream;
} MAP_STRUCT;
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
typedef struct{
	uint32 Offset;
	uint32 ID;
} DEP_RET, LID_RET;

typedef struct{
	uint32 Offset;
	uint32 Refrence;
	uint32 Count;
}REFLEX_RET, STR_ID;
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
typedef struct{
	uint8 R;
	uint8 G;
	uint8 B;
	uint8 OxFF;
}LightMapColour;
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
typedef struct{
	FILE *MainMenu;
	FILE *Shared;
	FILE *SharedSP;
}GLOBALMAPS;
///////////////////////////////////////
///////////////////////////////////////

#define	TAG_CLASS	0x01
#define	TAG_SIZE	0x02
#define	TAG_OFFSET	0x04
#define	TAG_ID		0x08
#define MAX_CHUNKS	0xFF
#define HaloFree(x) free(x)
#define isNotExtern(x)	((x>>28)&0xff)

#define	FindStringID(x,y)		y.StringTable+y.StringIndex[x]
#define FindFileName(x,y)		y.MetaTable+y.MetaIndex[x]
#define GetTagID(x,y)			y.ItemIndex[x].id
#define GetTagClass(x,y)		y.ItemIndex[x].tagclass
#define GetTagSize(x,y)			y.ItemIndex[x].size
#define GetTagOffset(x,y)		y.ItemIndex[x].offset-y.SecondaryMagic
#define TagIsThisClass(t,c,m)	strncmp(m.ItemIndex[t].tagclass,c,4)==0
GLOBALMAPS SharedMaps;

//map.c
#define FindTagByID(x) x&0xFFFF 

MAP_STRUCT OpenMap(char *path);
void	CloseMap(MAP_STRUCT Map);
int		InitHalo2(char *mainmenu, char *shared, char *sharedsp);
int		FindTag(char *tag, char *class,MAP_STRUCT Map);
int		FindTagByOffset(uint32 offset, MAP_STRUCT Map);
void	SetTagAttributes(	int tag, char *class,
							int size,uint32 offset,uint32 ID,
							uint8 flags, MAP_STRUCT Map);
void	WriteIndex(MAP_STRUCT Map);

//utils.c
char	*ClassEdianSwap(char *class);
int		isClass(char *class,MAP_STRUCT Map);
uint32	XORSum(uint32 *buffer,int len);
uint32	XORSign(MAP_STRUCT Map);
int		FindCoconuts(MAP_STRUCT Map);
void	FixMetaSize(MAP_STRUCT Map, int size);
int		isBSP(int tag, MAP_STRUCT Map);
int		isWithinBSP(int addy, MAP_STRUCT Map);
int		isLTMP(int tag, MAP_STRUCT Map);
int		isWithinLTMP(int addy, MAP_STRUCT Map);

//Reflexives.c
int		GetReflexiveAddressTag(MAP_STRUCT Map,int tag,uint32 offset);
int		GetReflexiveChunkCountTag(MAP_STRUCT Map,int tag,uint32 offset);
int		GetReflexiveAddress(MAP_STRUCT Map,uint32 offset);
int		GetReflexiveChunkCount(MAP_STRUCT Map,uint32 offset);
void	FixReflexive(int tag, uint32 offset, int pointer_tag, uint32 pointer_offset, MAP_STRUCT Map);

//shift.c
void	ShiftUnicodeStringTables(MAP_STRUCT Map, int offset);
void	ShiftSBSPBlock(MAP_STRUCT Map, int bsp,int offset);
void	ShiftScriptTable(MAP_STRUCT Map, int offset);
void	ShiftScriptIndex(MAP_STRUCT Map, int offset);
void	ShiftFileTable(MAP_STRUCT Map, int offset);
void	ShiftFileIndex(MAP_STRUCT Map, int offset);
void	ShiftIndex(MAP_STRUCT Map, int offset);
void	ShiftStringTable128(MAP_STRUCT Map, int offset);

//Animations.c
void	ShiftAnimations(MAP_STRUCT Map, int offset);

//Bitmaps.c
void	ShiftBitmaps(MAP_STRUCT Map, int bytes);

//Models.c
void	ShiftModels(MAP_STRUCT Map, int offset);

//Scanner.c
DEP_RET		*DependencyScan(int tag, MAP_STRUCT Map, int *count);
LID_RET		*LoneIDScan(int tag, MAP_STRUCT Map, int *count);
STR_ID		*StringIDScan(int tag, MAP_STRUCT Map,int *count);
REFLEX_RET	*ReflexiveScan(int tag, MAP_STRUCT Map,int *count);

//Dependencies.c
int		SwapDependency(char *tag, char *class, int tagtofix, uint32 offset, MAP_STRUCT Map);
int		SwapDependencyTag(int tag, int tagtofix, uint32 offset, MAP_STRUCT Map);

//LoneIDs.c
int		SwapLoneID(char *tag, char *class, int tagtofix, uint32 offset, MAP_STRUCT Map);
int		SwapLoneIDTag(int tag, int tagtofix, uint32 offset, MAP_STRUCT Map);

//Lightmaps.c
void			ShiftLightMapRaw(MAP_STRUCT Map, int lightmap, int offset);
LightMapColour	*GetLightMapColourStruct(MAP_STRUCT Map, int *count,int lightmap);
void			WriteLightMapColourStruct(MAP_STRUCT Map, int count, int lightmap,LightMapColour *colours);

//Weather.c
void	ShiftWeatherRaw(MAP_STRUCT Map,int tag, int offset);
#ifdef __cplusplus
}
#endif

#endif /*__LIBHALO_2__*/
