/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	Written by: Joshua Horacsek
 *	Mon Jan  2 15:37:54 2006
 */
#include "halo2.h"
int InitHalo2(char *mainmenu, char *shared, char *sharedsp){
	SharedMaps.MainMenu	=	fopen(mainmenu,"rb");
	SharedMaps.Shared	=	fopen(shared,"rb");
	SharedMaps.SharedSP	=	fopen(sharedsp,"rb");
	if(	SharedMaps.MainMenu		==	NULL ||
		SharedMaps.Shared		==	NULL ||
		SharedMaps.SharedSP		==	NULL)
		return -1;
	return 0;
}
MAP_STRUCT OpenMap(char *path){
	MAP_STRUCT Map;
	uint32 tempOffset, tempChunkCount,i;
	if((Map.Stream=fopen(path,"rb+"))==NULL){
		return Map;
	}
	/*
	 Save some info for later...
	*/
	Map.MapPath=malloc(strlen(path)+1);
	strcpy(Map.MapPath,path);
	/*
	 Now that we know the Map exists, we can allocate memory
	 for Map's Initial Data structures
	*/
	Map.Header=calloc(1,sizeof(Halo2Map));
	Map.Index=calloc(1,sizeof(IndexHeader));
	
	fread(Map.Header,1,sizeof(Halo2Map),Map.Stream);
	fseek(Map.Stream,Map.Header->indexOffset,0);
	fread(Map.Index,1,sizeof(IndexHeader),Map.Stream);
	
	//Some Simple Error Checking
	if(!strncmp(Map.Header->header,"head",4)){
		fclose(Map.Stream);
		Map.Stream=NULL;
		return Map;
	}
	if(Map.Header->Version != 8){
		fclose(Map.Stream);
		Map.Stream=NULL;
		return Map;
	}
	//Allocate Memory for tag and item indices
	Map.TagIndex=calloc(Map.Index->tagcount,sizeof(IndexTag));
	Map.ItemIndex=calloc(Map.Index->itemcount,sizeof(IndexItem));

	Map.TagIndexOffset	=	ftell(Map.Stream);
	fread(Map.TagIndex,Map.Index->tagcount,sizeof(IndexTag),Map.Stream);
	Map.ItemIndexOffset	=	ftell(Map.Stream);
	fread(Map.ItemIndex,Map.Index->itemcount,sizeof(IndexItem),Map.Stream);
	
	//Allocate space for then Load the Meta Filename and Script Tables
	Map.StringTable=malloc(Map.Header->sizeOfScriptReference);
	Map.MetaTable=malloc(Map.Header->fileTableSize);
	
	fseek(Map.Stream,Map.Header->offsetToScriptReferenceStrings,0);
	fread(Map.StringTable,Map.Header->sizeOfScriptReference,1,Map.Stream);
	
	fseek(Map.Stream,Map.Header->fileTableOffset,0);
	fread(Map.MetaTable,Map.Header->fileTableSize,1,Map.Stream);
	
	//Allocate space for then Load the Meta Filename and Script Indices
	Map.StringIndex=calloc(Map.Header->scriptReferenceCount,sizeof(uint32));
	Map.MetaIndex=calloc(Map.Header->fileCount,sizeof(uint32));
	
	fseek(Map.Stream,Map.Header->offsetToScriptReferenceIndex,0);
	fread(Map.StringIndex,Map.Header->scriptReferenceCount,sizeof(uint32),Map.Stream);
	
	fseek(Map.Stream,Map.Header->filesIndex,0);
	fread(Map.MetaIndex,Map.Header->fileCount,sizeof(uint32),Map.Stream);

	strcpy(Map.MapName,Map.Header->mapname);
	strcpy(Map.scenarioname,Map.Header->scenarioname);
	//Calculate Magic Values	
	Map.PrimaryMagic	=	Map.Index->indexmagic-Map.Header->indexOffset+sizeof(IndexHeader);
	Map.SecondaryMagic	=	Map.ItemIndex[0].offset-(Map.Header->indexOffset+Map.Header->indexSize);
	Map.MetaCount		=	Map.Index->itemcount;
	Map.StringCount		=	Map.Header->scriptReferenceCount;
	//Finding Unicode Strings...
	
	//Finding SBSP
	Map.bspCount		=	Map.ltmpCount = 0;
	/*We set them to 0 while we attempt to get their info
	 *or else functions in reflexives.c will check if the
	 *tag is a BSP or LTMP tag. And since there are none
	 *well...
	 */
	tempOffset			=	GetReflexiveAddressTag(Map,3,528);
	Map.bspCount		=	Map.ltmpCount = GetReflexiveChunkCountTag(Map,3,528);
	Map.BSPs			=	calloc(Map.bspCount,sizeof(BSP_Data));
	Map.LTMPs			=	calloc(Map.ltmpCount,sizeof(LTMP_Data));
	
	//memset(Map.BSPs,0,(Map.bspCount*sizeof(BSP_Data)));
	//memset(Map.LTMPs,0,(Map.ltmpCount*sizeof(LTMP_Data)));
	
	for(i=0;i<Map.bspCount;i++){
		fseek(Map.Stream,tempOffset+(i*68),0);	//Locate The Info we want
		Map.BSPs[i].OffsetToOffset=tempOffset+(i*68);
		fread(&Map.BSPs[i].Offset,sizeof(uint32),1,Map.Stream);
		fread(&Map.BSPs[i].Size,sizeof(uint32),1,Map.Stream);
		fread(&Map.BSPs[i].Magic,sizeof(uint32),1,Map.Stream);
		
		Map.BSPs[i].Magic	-=	Map.BSPs[i].Offset;	//Calculate the BSP magic
		fseek(Map.Stream,tempOffset+(i*68)+20,0);
		
		fread(&Map.BSPs[i].TagID,sizeof(uint32),1,Map.Stream);
		Map.BSPs[i].BSPBlockSize	=	Map.BSPs[i].Size;
		Map.BSPs[i].Tag				=	FindTagByID(Map.BSPs[i].TagID);
	}
	//Yeah Lightmaps! Wouldn't have found this out without the 
	// big Iron_Forge in the sky
	for(i=0;i<Map.ltmpCount;i++){
		tempOffset			=	GetReflexiveAddressTag(Map,Map.BSPs[i].Tag,4);
		Map.LTMPs[i].Size	=	Map.BSPs[i].Size-(tempOffset-Map.BSPs[i].Offset);
		Map.BSPs[i].Size	=	tempOffset-Map.BSPs[i].Offset;
		Map.LTMPs[i].Offset	=	tempOffset;
		Map.LTMPs[i].Magic	=	Map.BSPs[i].Magic;
		Map.LTMPs[i].Tag	=	Map.BSPs[i].Tag-2;
	}
	return Map;
}
void CloseMap(MAP_STRUCT Map){
#ifdef	AUTOMATIC_RESIGN
//	XORSign(Map);
#endif	/*AUTOMATIC_RESIGN*/
	fclose(Map.Stream);
	free(Map.MetaIndex);
	free(Map.MetaTable);
	free(Map.StringIndex);
	free(Map.StringTable);
	free(Map.BSPs);
	free(Map.LTMPs);
	free(Map.Header);
	free(Map.Index);

	Map.Stream=NULL;
}

int FindTag(char *tag, char *class,MAP_STRUCT Map){
	int i;
	char *Class=ClassEdianSwap(class);
	for(i=0;i<Map.MetaCount;i++){
		if(strcmp(FindFileName(i,Map),tag)==0){
			if(strncmp(Class,Map.ItemIndex[i].tagclass,4)==0){
				HaloFree(Class);
				return i;
			}
		}
	}
	HaloFree(Class);
	return -1;
}

int FindTagByOffset(uint32 offset, MAP_STRUCT Map){
	int i;
	for(i=0;i<Map.MetaCount;i++){
		if(	Map.ItemIndex[i].offset-Map.SecondaryMagic >= offset &&
			Map.ItemIndex[i].offset+Map.ItemIndex[i].size-Map.SecondaryMagic<=offset){
			return i;
		}
	}
	return -1;
}
/* Script/MetaFileName Operations
*/
void RenameTag(char *Name,int tag, MAP_STRUCT Map){
	//Map.
}

//Returns the full possible length of the meta file name
int TagNameLen(int Tag, MAP_STRUCT Map){
}


void SetTagAttributes(	int tag, char *class,
						int size,uint32 offset,uint32 ID,
						uint8 flags, MAP_STRUCT Map){
	char *Class=ClassEdianSwap(class);
	Map.ItemIndex[tag].size		= (flags & TAG_SIZE==TAG_SIZE)?size:Map.ItemIndex[tag].size;
	Map.ItemIndex[tag].offset	= (flags & TAG_OFFSET==TAG_OFFSET)?offset+Map.SecondaryMagic:Map.ItemIndex[tag].offset;
	Map.ItemIndex[tag].id		= (flags & TAG_ID==TAG_ID)?ID:Map.ItemIndex[tag].id;
	if(flags & TAG_CLASS==TAG_CLASS)
		strncpy(Map.ItemIndex[tag].tagclass,Class,4);
	HaloFree(Class);
	WriteIndex(Map);
}

void WriteIndex(MAP_STRUCT Map){
	fseek(	Map.Stream,Map.ItemIndexOffset,0);
	fwrite(	Map.ItemIndex,Map.Index->itemcount,sizeof(IndexItem),Map.Stream);
}
